| Safe Haskell | None |
|---|---|
| Language | GHC2021 |
Futhark.Version
Description
This module exports version information about the Futhark compiler.
Synopsis
- version :: Version
- versionString :: Text
| Safe Haskell | None |
|---|---|
| Language | GHC2021 |
Futhark.Version
Description
This module exports version information about the Futhark compiler.